@@ -1,6 +1,6 @@
 /*
- *  Eukleides  version 0.9.2
- *  Copyright (c) Christian Obrecht 2000-2002
+ *  Eukleides  version 1.0.3
+ *  Copyright (c) Christian Obrecht 2000-2004
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -116,6 +116,7 @@ double eccentricity (_conic * C)
       case HYPERBOLA:
 	  return sqrt (1 + C->b * C->b / (C->a * C->a));
       }
+    return 1;
 }
 
 double height (_point * A, _point * B, _point * C)
@@ -194,6 +195,11 @@ double T_angle (_point * A, _point * B, _point * C)
 		   angle (A->x - B->x, A->y - B->y));
 }
 
+double C_argument (_circle * C, _point * A)
+{
+    return angle (A->x - C->x, A->y - C->y);
+}
+
 double Co_argument (_conic * C, _point * A)
 {
     double c0, s0, x, y, z;
@@ -1152,8 +1158,18 @@ void foci (symrec * s1, symrec * s2, _conic * C)
       }
 }
 
-void T_assignment (symrec * s1, symrec * s2, double x, double y, double b,
-		   double h, double p, double l, double a)
+void
+T_assignment (symrec * s, double x, double y, 
+	      double h, double p, double l, double a)
+{
+    P_assignment (s,
+		  P_new (x + l * (p * Cos (a) - h * Sin (a)),
+			 y + l * (h * Cos (a) + p * Sin (a))));
+}
+
+void
+T_assignments (symrec * s1, symrec * s2, double x, double y, double b,
+	      double h, double p, double l, double a)
 {
     P_assignment (s1, P_new (x + l * b * Cos (a), y + l * b * Sin (a)));
     P_assignment (s2,
@@ -1161,31 +1177,63 @@ void T_assignment (symrec * s1, symrec * s2, double x, double y, double b,
 			 y + l * (h * Cos (a) + p * Sin (a))));
 }
 
+void T_check (symrec * s1, symrec * s2)
+{
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (6, 0));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+6, s1->object.point->y));
+}
+  
 void T_scalenes (symrec * s1, symrec * s2, symrec * s3, double l, double a)
 {
     if (s1->type != POINT)
 	P_assignment (s1, P_new (0, 0));
-    T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, 1,
+    T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, 1,
 		  .61237244, .375, l, a);
 }
 
-void T_N_triangle (symrec * s1, symrec * s2, symrec * s3, double l1,
-		   double l2, double l3, double a)
+void
+T_3N_triangle (symrec * s1, symrec * s2, symrec * s3, double l1, double l2,
+	      double l3, double a)
 {
     double h, p;
+    
     if (ZERO (l1) || (l1 > l2 + l3) || (l2 > l1 + l3) || (l3 > l1 + l2))
 	yyerror ("invalid length.");
     if (s1->type != POINT)
 	P_assignment (s1, P_new (0, 0));
     h = sqrt ((l1 + l2 + l3) * (l1 + l3 - l2) * (l1 + l2 - l3) * (l2 + l3 - l1))
-	     / (2 * l1);
-    p = sqrt (l3 * l3 - h * h) * ((l2 * l2 > (l1 * l1 + l3 * l3)) ? -1 : 1);
-    T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, l1, h, p,
+	 / (2 * l1);
+    p = sqrt (fabs(l3 * l3 - h * h))
+	 * ((l2 * l2 > (l1 * l1 + l3 * l3)) ? -1 : 1);
+    T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, l1, h, p,
 		  1, a);
 }
 
-void T_A_triangle (symrec * s1, symrec * s2, symrec * s3, double l, double a1,
-		   double a2, double a)
+void
+T_2N_triangle (symrec * s1, symrec * s2, symrec * s3, double l2, double l3)
+{
+    double a, l1, h, p;
+   
+    T_check (s1, s2);
+    l1 = PP_distance(s1->object.point,s2->object.point);
+    if (ZERO (l1) || (l1 > l2 + l3) || (l2 > l1 + l3) || (l3 > l1 + l2))
+	yyerror ("invalid length.");
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);
+    h = sqrt ((l1 + l2 + l3) * (l1 + l3 - l2) * (l1 + l2 - l3) * (l2 + l3 - l1))
+	 / (2 * l1);
+    p = sqrt (fabs(l3 * l3 - h * h))
+	 * ((l2 * l2 > (l1 * l1 + l3 * l3)) ? -1 : 1);
+    T_assignment (s3, s1->object.point->x, s1->object.point->y, h, p, 1, a);
+}
+
+void
+T_2A_triangle (symrec * s1, symrec * s2, symrec * s3, double l, double a1,
+	      double a2, double a)
 {
     _line *tmp1, *tmp2;
 
@@ -1201,53 +1249,168 @@ void T_A_triangle (symrec * s1, symrec * s2, symrec * s3, double l, double a1,
     P_assignment (s3, P_LL_intersection (tmp1, tmp2));
 }
 
-void T_right (symrec * s1, symrec * s2, symrec * s3, double l1, double l2,
-	      double a)
+void
+T_A_triangle (symrec * s1, symrec * s2, symrec * s3, double a1, double a2)
+{
+    _line *tmp1, *tmp2;
+    
+    double a;
+    if (ZERO (Sin (a1 + a2)))
+	yyerror ("invalid angle.");
+    T_check (s1, s2);
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);    
+    tmp1 = L_PA_new (s1->object.point, a1 + a);
+    tmp2 = L_PA_new (s2->object.point, format (180 - a2 + a));
+    P_assignment (s3, P_LL_intersection (tmp1, tmp2));
+}
+
+void
+T_2N_right (symrec * s1, symrec * s2, symrec * s3, double l1, double l2,
+	    double a)
 {
     if (s1->type != POINT)
 	P_assignment (s1, P_new (0, 0));
-    T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, l1, l2, 0,
+    T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, l1, l2, 0,
 		  1, a);
 }
 
-void T_A_isosceles (symrec * s1, symrec * s2, symrec * s3, double l1,
-		    double a1, double a)
+void
+T_N_right (symrec * s1, symrec * s2, symrec * s3, double l2)
+{
+    double a;
+    T_check (s1, s2);
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);
+    T_assignment (s3, s1->object.point->x, s1->object.point->y, l2, 0,
+		  1, a);
+}
+
+void
+T_NA_isosceles (symrec * s1, symrec * s2, symrec * s3, double l1, double a1,
+	       double a)
 {
     if (ZERO (Cos (a1)))
 	yyerror ("invalid angle.");
-    else
-      {
-	  if (s1->type != POINT)
-	      P_assignment (s1, P_new (0, 0));
-	  T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, l1,
-			l1 * Tan (a1) / 2, l1 / 2, 1, a);
-      }
+    else {
+	if (s1->type != POINT)
+	    P_assignment (s1, P_new (0, 0));
+	T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, l1,
+		      l1 * Tan (a1) / 2, l1 / 2, 1, a);
+    }
 }
 
-void T_N_isosceles (symrec * s1, symrec * s2, symrec * s3, double l1,
-		    double l2, double a)
+void
+T_A_isosceles (symrec * s1, symrec * s2, symrec * s3, double a1)
+{
+    double a, l1;
+
+    if (ZERO (Cos (a1)))
+	yyerror ("invalid angle.");
+    else {
+	T_check (s1, s2);
+	a = angle (s2->object.point->x-s1->object.point->x,
+		   s2->object.point->y-s1->object.point->y);
+	l1 = PP_distance(s1->object.point,s2->object.point);
+	T_assignment (s3, s1->object.point->x, s1->object.point->y,
+		      l1 * Tan (a1) / 2, l1 / 2, 1, a);
+    }
+}
+
+void
+T_2N_isosceles (symrec * s1, symrec * s2, symrec * s3, double l1, double l2,
+	       double a)
 {
     if (l1 > 2 * l2)
 	yyerror ("invalid length.");
-    else
-      {
-	  if (s1->type != POINT)
-	      P_assignment (s1, P_new (0, 0));
-	  T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, l1,
-			sqrt (l2 * l2 - l1 * l1 / 4), l1 / 2, 1, a);
-      }
+    else {
+	if (s1->type != POINT)
+	    P_assignment (s1, P_new (0, 0));
+	T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, l1,
+		      sqrt (l2 * l2 - l1 * l1 / 4), l1 / 2, 1, a);
+    }
+}
+
+void
+T_N_isosceles (symrec * s1, symrec * s2, symrec * s3, double l2)
+{
+    double a, l1;
+
+    if (l1 > 2 * l2)
+	yyerror ("invalid length.");
+    else {
+	T_check (s1, s2);
+	a = angle (s2->object.point->x-s1->object.point->x,
+		   s2->object.point->y-s1->object.point->y);
+	l1 = PP_distance(s1->object.point,s2->object.point);
+	T_assignment (s3, s1->object.point->x, s1->object.point->y,
+		      sqrt (l2 * l2 - l1 * l1 / 4), l1 / 2, 1, a);
+    }
+}
+
+void T_E_equilateral (symrec * s1, symrec * s2, symrec * s3)
+{
+    double l, a;
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);
+    l = PP_distance(s1->object.point,s2->object.point);
+    T_assignment (s3, s1->object.point->x, s1->object.point->y,
+	      .866025, .5, l, a);
 }
 
 void T_equilateral (symrec * s1, symrec * s2, symrec * s3, double l, double a)
 {
     if (s1->type != POINT)
 	P_assignment (s1, P_new (0, 0));
-    T_assignment (s2, s3, s1->object.point->x, s1->object.point->y, 1,
-		  .866025, .5, l, a);
+    T_assignments (s2, s3, s1->object.point->x, s1->object.point->y, 1,
+	      .866025, .5, l, a);
 }
 
-void Q_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
-		      double l1, double l2, double a1, double a)
+void
+Q_E_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4)
+{
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (5, 0));
+	P_assignment (s3, P_new (6.0352, 3.8637));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+5, s1->object.point->y));
+    if (s2->type != POINT || s3->type != POINT)
+	P_assignment (s3, s2->object.point);
+    P_assignment (s4, P_new (
+	s1->object.point->x + (s3->object.point->x - s2->object.point->x),
+	s1->object.point->y + (s3->object.point->y - s2->object.point->y)));
+}
+
+void
+Q_S_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
+		   double l2, double a1)
+{
+    double l1, a, xu, xv, yu, yv;
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (5, 0));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+5, s1->object.point->y));
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);
+    l1 = PP_distance(s1->object.point,s2->object.point);
+    xu = l1 * Cos (a);
+    xv = l2 * Cos (a + a1);
+    yu = l1 * Sin (a);
+    yv = l2 * Sin (a + a1);
+    P_assignment (s3,
+		  P_new (s1->object.point->x + xu + xv,
+			 s1->object.point->y + yu + yv));
+    P_assignment (s4,
+		  P_new (s1->object.point->x + xv, s1->object.point->y + yv));
+}
+
+void
+Q_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
+		 double l1, double l2, double a1, double a)
 {
     double xu, xv, yu, yv;
 
@@ -1266,8 +1429,9 @@ void Q_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
 		  P_new (s1->object.point->x + xv, s1->object.point->y + yv));
 }
 
-void Q_V_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
-			_vector * u, _vector * v)
+void
+Q_V_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
+		   _vector * u, _vector * v)
 {
     if (s1->type != POINT)
 	P_assignment (s1, P_new (0, 0));
@@ -1282,11 +1446,65 @@ void Q_V_parallelogram (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
 			 s1->object.point->y + v->y));
 }
 
+void
+Q_E_rectangle (symrec * s1, symrec * s2, symrec * s3, symrec * s4)
+{
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (6, 0));
+	P_assignment (s3, P_new (6, 3.7082));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+6, s1->object.point->y));
+    if (s3->type != POINT)
+	P_assignment (s3, s2->object.point);
+    P_assignment (s4, P_new (
+	s1->object.point->x + (s3->object.point->x - s2->object.point->x),
+	s1->object.point->y + (s3->object.point->y - s2->object.point->y)));
+}
+
+void
+Q_S_rectangle (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
+		   double l2)
+{
+    double l1, a, xu, xv, yu, yv;
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (6, 0));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+6, s1->object.point->y));
+    a = angle (s2->object.point->x-s1->object.point->x,
+	       s2->object.point->y-s1->object.point->y);
+    l1 = PP_distance(s1->object.point,s2->object.point);
+    xu = l1 * Cos (a);
+    xv = -l2 * Sin (a);
+    yu = l1 * Sin (a);
+    yv = l2 * Cos (a);
+    P_assignment (s3,
+		  P_new (s1->object.point->x + xu + xv,
+			 s1->object.point->y + yu + yv));
+    P_assignment (s4,
+		  P_new (s1->object.point->x + xv, s1->object.point->y + yv));
+}
+
+void
+Q_square (symrec * s1, symrec * s2, symrec * s3, symrec * s4)
+{
+    if (s1->type != POINT) {
+	P_assignment (s1, P_new (0, 0));
+	P_assignment (s2, P_new (4, 0));
+    }
+    if (s2->type != POINT)
+	P_assignment (s2, P_new (s1->object.point->x+4, s1->object.point->y));
+    Q_S_rectangle (s1, s2, s3, s4,
+	    PP_distance(s1->object.point,s2->object.point));
+}
+
 void pentagon (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
 	       symrec * s5, _point * O, double r, double a)
 {
-    if (s1->type != POINT)
-	P_assignment (s1, P_new (O->x + r * Cos (a), O->y + r * Sin (a)));
+    P_assignment (s1, P_new (O->x + r * Cos (a), O->y + r * Sin (a)));
     P_assignment (s2,
 		  P_new (O->x + r * Cos (a + 72), O->y + r * Sin (a + 72)));
     P_assignment (s3,
@@ -1300,8 +1518,7 @@ void pentagon (symrec * s1, symrec * s2, symrec * s3, symrec * s4,
 void hexagon (symrec * s1, symrec * s2, symrec * s3, symrec * s4, symrec * s5,
 	      symrec * s6, _point * O, double r, double a)
 {
-    if (s1->type != POINT)
-	P_assignment (s1, P_new (O->x + r * Cos (a), O->y + r * Sin (a)));
+    P_assignment (s1, P_new (O->x + r * Cos (a), O->y + r * Sin (a)));
     P_assignment (s2,
 		  P_new (O->x + r * Cos (a + 60), O->y + r * Sin (a + 60)));
     P_assignment (s3,
